/*
 * Jalview - A Sequence Alignment Editor and Viewer (Version 2.6)
 * Copyright (C) 2010 J Procter, AM Waterhouse, G Barton, M Clamp, S Searle
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 * 
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 */
package jalview.gui;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

import java.awt.event.*;
import java.io.*;

import jalview.api.SequenceStructureBinding;
import jalview.bin.Cache;
import jalview.datamodel.*;
import jalview.gui.ViewSelectionMenu.ViewSetProvider;
import jalview.structure.*;
import jalview.io.*;
import jalview.schemes.*;
import fr.orsay.lri.varna.VARNAPanel;
import fr.orsay.lri.varna.exceptions.ExceptionFileFormatOrSyntax;
import fr.orsay.lri.varna.exceptions.ExceptionNonEqualLength;
import fr.orsay.lri.varna.exceptions.ExceptionUnmatchedClosingParentheses;
import fr.orsay.lri.varna.interfaces.InterfaceVARNAListener;
import fr.orsay.lri.varna.models.VARNAConfig;
import fr.orsay.lri.varna.models.rna.RNA;


public class AppVarna extends JInternalFrame implements InterfaceVARNAListener// implements Runnable,SequenceStructureBinding, ViewSetProvider

{
  AppVarnaBinding vab;

  VARNAPanel varnaPanel;
  
  public String name;
  
  /*public AppVarna(){
	  vab = new AppVarnaBinding(); 
	  initVarna();
  }*/
  
  public AppVarna(String seq,String struc,String name){
	  ArrayList<RNA> rnaList = new ArrayList<RNA>();
	  RNA rna1 = new RNA(name);
	  try {
		  rna1.setRNA(seq,struc);
	  } catch (ExceptionUnmatchedClosingParentheses e2) {
		e2.printStackTrace();
	  } catch (ExceptionFileFormatOrSyntax e3) {
		e3.printStackTrace();
	  }
	  rnaList.add(rna1);
	  rnaList.add(trimRNA(rna1));
	  
	  vab = new AppVarnaBinding(rnaList);
	  //vab = new AppVarnaBinding(seq,struc);
	  //System.out.println("Hallo: "+name);
	  this.name=name;
	  initVarna();	  
  }
  
  public void initVarna(){
	  //vab.setFinishedInit(false);
	  varnaPanel=vab.get_varnaPanel();
	  setBackground(Color.white);
	  JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,true,vab.getListPanel(),varnaPanel);
	  getContentPane().setLayout(new BorderLayout());
	  getContentPane().add(split, BorderLayout.CENTER);
	  //getContentPane().add(vab.getTools(), BorderLayout.NORTH);	
	  varnaPanel.addVARNAListener(this);
	  jalview.gui.Desktop.addInternalFrame(this,"VARNA -"+name,getBounds().width, getBounds().height);
	  this.pack();
	  showPanel(true);
  }
  
  public RNA trimRNA(RNA rna){
	  RNA rnaTrim = new RNA(rna.getName()+"_trim");
	  String seq=rna.getSeq();
	  String struc=rna.getStructDBN();
	  String newSeq="";
	  String newStruc="";
	  for(int i=0;i<seq.length();i++){
		  if(seq.substring(i, i+1).compareTo("-")!=0){
			  newSeq=newSeq+seq.substring(i, i+1);
			  newStruc=newStruc+struc.substring(i,i+1);
		  }
	  }
	  try {
		rnaTrim.setRNA(newSeq,newStruc);
	} catch (ExceptionUnmatchedClosingParentheses e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} catch (ExceptionFileFormatOrSyntax e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	//System.out.println(newSeq+"\t"+newStruc);
	return rnaTrim;
  }

  public void showPanel(boolean show){
	  this.setVisible(show);
  }
  
  private boolean _started = false;

  public void run(){
	  _started = true;
	  
	  try
      {
        initVarna();
      } catch (OutOfMemoryError oomerror)
      {
        new OOMWarning("When trying to open the Varna viewer!", oomerror);
      } catch (Exception ex)
      {
        Cache.log.error("Couldn't open Varna viewer!", ex);
      }
  }

@Override
public void onLayoutChanged() {
	// TODO Auto-generated method stub
	
}

@Override
public void onUINewStructure(VARNAConfig v, RNA r) {
	//TODO _rnaList.add(v, r,"",true);
	// TODO Auto-generated method stub
	
}

@Override
public void onWarningEmitted(String s) {
	// TODO Auto-generated method stub
	
}
  

}
